/// <reference path="../js/jquery-1.12.3.min.js" />
/// <reference path="../js/knockout-3.4.0.js" />
/// <reference path="../js/dx.all.js" />

(function (ko) {
    // Wrap ko.observable and ko.observableArray
    var methods = ['observable', 'observableArray'];

    ko.utils.arrayForEach(methods, function (method) {
        var saved = ko[method];

        ko[method] = function (initialValue, options) {
            options = options || {};

            var key = options.persist;

            // Load existing value if set
            if (key && localStorage.hasOwnProperty(key)) {
                try {
                    initialValue = JSON.parse(localStorage.getItem(key))
                } catch (e) { };
            }

            // Create observable from saved method
            var observable = saved(initialValue);

            // Subscribe to changes, and save to localStorage
            if (key) {
                observable.subscribe(function (newValue) {
                    localStorage.setItem(key, ko.toJSON(newValue));
                });
            };

            return observable;
        }
    })
})(ko);

(function () {
    IntelliServe_Dxtreme.currentUser = {
        UserName: ko.observable("", { persist: 'UserName' }),
        Password: ko.observable("", { persist: 'Password' }),
        IsAuthenticated: ko.observable("false", { persist: 'IsAuthenticated' }),
        DisplayName: ko.observable("Joe Soap", { persist: 'DisplayName' }),
        UserImage: ko.observable("images/noavatar.jpg", { persist: 'UserImage' }),
        LocationServicesEnabled: ko.observable(true, { persist: 'LocationServicesEnabled' }),
        MapService: ko.observable(true, { persist: 'MapService' }),
        MapServiceDescription: ko.observable("bing", { persist: 'MapServiceDescription' }),
        Database: ko.observable("", { persist: 'Database' }),

        login: function (UserName, Password, CallbackMethod) {
            this.UserName($.trim(UserName));
            this.Password($.trim(Password));
            IntelliServe_Dxtreme.dbDataLayer.Instance().IsUserAllowed(CallbackMethod);
        },

        clear: function () {
            this.UserName("");
            this.Password("");
            this.Database("");
            this.IsAuthenticated("false");
            this.DisplayName("Joe Soap");
            this.UserImage("images/noavatar.jpg");
        },

        fromJS: function (data) {
            if (data) {
                this.UserName(data.UserName);
                this.Password(data.Password);
                this.IsAuthenticated(data.IsAuthenticated);
                this.DisplayName(data.DisplayName);
                this.UserImage(data.UserImage);
            }
        }
    };
})();

(function() {
    var isWinJS = "WinJS" in window;
    var endpointSelector = new DevExpress.EndpointSelector(IntelliServe_Dxtreme.config.endpoints);
    var serviceConfig = $.extend(true, {}, IntelliServe_Dxtreme.config.services, {
        db: {
            url: endpointSelector.urlFor("db"),
            jsonp: !isWinJS,
            errorHandler: handleServiceError,
            beforeSend: function (sender) {
                sender.params.UserName = IntelliServe_Dxtreme.currentUser.UserName();
                sender.params.Password = IntelliServe_Dxtreme.currentUser.Password();
                sender.params.Database = IntelliServe_Dxtreme.currentUser.Database();
            }
        }
    });

    function handleServiceError(error) {
        DevExpress.ui.notify(error.message, 'Error', 3000);
    }

    // Enable partial CORS support for IE < 10    
    $.support.cors = true;
    
    IntelliServe_Dxtreme.db = new DevExpress.data.ODataContext(serviceConfig.db);

}());
