﻿var AppUtils = (function () {
    //https://stackoverflow.com/questions/37206408/opening-a-pdf-in-cordova-javascript        
    // get the application directory. in this case only checking for Android and iOS
    function localFilePath(filename) {
        if (device.platform.toLowerCase() === 'android') {
            return cordova.file.externalDataDirectory + filename;
        } else {
            return cordova.file.dataDirectory + filename;
        }
    }

    // FileWritter class
    function FileWritter(filename) {
        this.fileName = filename;
        this.filePath = localFilePath(filename);
    }

    // decode base64 encoded data and save it to file
    FileWritter.prototype.saveBase64ToBinary = function (data, ok, fail) {
        var contentType = "application/" + this.fileName.split('.').pop();
        var byteData = b64toBlob(data, contentType);

        this.saveFile(byteData, ok, fail);
    }

    function b64toBlob(b64Data, contentType, sliceSize) {
        contentType = contentType || '';
        sliceSize = sliceSize || 512;

        var byteCharacters = atob(b64Data);
        var byteArrays = [];

        for (var offset = 0; offset < byteCharacters.length; offset += sliceSize) {
            var slice = byteCharacters.slice(offset, offset + sliceSize);

            var byteNumbers = new Array(slice.length);
            for (var i = 0; i < slice.length; i++) {
                byteNumbers[i] = slice.charCodeAt(i);
            }

            var byteArray = new Uint8Array(byteNumbers);

            byteArrays.push(byteArray);
        }

        var blob = new Blob(byteArrays, { type: contentType });
        return blob;
    }

    // save file to storage using cordova
    FileWritter.prototype.saveFile = function (data, ok, fail) {
        this.fileData = data;

        var path = this.filePath.substring(0, this.filePath.lastIndexOf('/'));

        var that = this;

        // Write file on local system
        window.resolveLocalFileSystemURL(path, function (directoryEntry) {
            var options = { create: true };

            directoryEntry.getFile(that.fileName, options, function (file) {
                file.createWriter(function (writer) {
                    writer.onwriteend = function (event) {
                        if (typeof ok === 'function') {
                            ok(event);
                        }
                    };

                    writer.write(that.fileData);
                }, fail);
            }, fail);

        }, fail);
    };

    // open InApp Browser to view file
    function viewFile(filename) {
        var path = localFilePath(filename);

        window.resolveLocalFileSystemURL(path, function (entry) {
            if (device.platform.toLowerCase() === 'android') {
                window.open(entry.toURL(), "_system", "location=no,hardwareback=yes");
            } else {
                window.open(entry.toURL(), "_blank", "location=no,hardwareback=yes");
            }
        }, function (error) {
            DevExpress.ui.notify("File Error - " + error.code, 'Warning', 3000);
        });
    }

    return {
        FileWritter: FileWritter,
        localFilePath: localFilePath,
        viewFile: viewFile
    }
})();