IntelliServe_Dxtreme.JobCardDetails = function (params, viewInfo) {
    "use strict";

    var id = params.id,
        v_jobcards_mobile = new IntelliServe_Dxtreme.v_Jobcards_MobileViewModel(), actionSheetData, actionSheetVisible = ko.observable(false), showActionSheet,
        joblogactiontext = ko.observable(), joblogaction = ko.observable(),
        isReady = $.Deferred(), shouldReload = false,
        dataSourceItemsObservable = ko.observable(),
        dataSourceItems, dataSourceEquipmentsObservable = ko.observable(), dataSourceEquipments;

    function handleViewShowing() {
        IntelliServe_Dxtreme.db.v_Jobcards_Mobile.byKey(id).done(function (data) {
            v_jobcards_mobile.fromJS(data);

            if (v_jobcards_mobile.JobCardLogID() === null) {
                joblogactiontext("Start");
                joblogaction("JobCardStart/" + id);
            } else {
                joblogactiontext("Finish");
                joblogaction("JobCardEnd/" + id + '|' + v_jobcards_mobile.JobCardLogID());
            }

            isReady.resolve();
        });

        if (!dataSourceItemsObservable() | !dataSourceEquipmentsObservable()) {
            dataSourceItemsObservable(dataSourceItems);
            dataSourceEquipmentsObservable(dataSourceEquipments);
            dataSourceItems.load().always(function () {
                dataSourceEquipments.load().always(function () {
                    isReady.resolve();
                });
            });
        }
        else if (shouldReload) {
            refreshList();
        }
    }

    actionSheetData = [
        { text: "Add Equipment", click: function () { IntelliServe_Dxtreme.app.navigate("JobCardEquipmentAdd/" + id); } },
        { text: "Add Item", click: function () { IntelliServe_Dxtreme.app.navigate("JobCardItemAdd/" + id); } },
        { text: "Add Note", click: function () { IntelliServe_Dxtreme.app.navigate("JobCardNoteAdd/" + id); } },
        { text: "Client Details", click: function () { IntelliServe_Dxtreme.app.navigate("ClientDetails/" + v_jobcards_mobile.ClientOid()); } },
        { text: "Check Lists", click: function () { IntelliServe_Dxtreme.app.navigate("JobCardChecklist/" + id); } },
        { text: "Navigate", click: function () { launchNavigator(); } },
        { text: "Photo", click: function () { capturePhoto(); } },
        { text: "Signature", click: function () { IntelliServe_Dxtreme.app.navigate("JobCardSignature/" + id); } },
        { text: "Change Status", click: function () { IntelliServe_Dxtreme.app.navigate("JobCardStatus/" + id); } },
        { text: "Attachments", click: function () { IntelliServe_Dxtreme.app.navigate("JobCardAttachments/" + id); } }
    ];

    function capturePhoto() {
        // Take picture using device camera and retrieve image as base64-encoded string
        navigator.camera.getPicture(onPhotoDataSuccess, onPhotoDataFail, {
            quality: 80,
            destinationType: Camera.DestinationType.DATA_URL,
            encodingType: Camera.EncodingType.JPEG,
            allowEdit: false,
            popoverOptions: CameraPopoverOptions,
            correctOrientation: true,
            sourceType: Camera.PictureSourceType.CAMERA,
            targetWidth: 640,
            targetHeight: 480,
            saveToPhotoAlbum: false
        });
    }

    function launchNavigator() {
        if (device.platform.toLowerCase() === 'android') {
            window.location = "geo:0,0?q=" + v_jobcards_mobile.Latitude() + "," + v_jobcards_mobile.Longitude();
        } else {
            window.location = "maps://?q=" + v_jobcards_mobile.Latitude() + "," + v_jobcards_mobile.Longitude();
        }
    }

    function mapErrorHandler(error) {
        switch (error.code) {
            case error.PERMISSION_DENIED:
                DevExpress.ui.notify('User denied the request for Geolocation', 'error', 3000);
                break;
            case error.POSITION_UNAVAILABLE:
                DevExpress.ui.notify('Location information is unavailable', 'error', 3000);
                break;
            case error.TIMEOUT:
                DevExpress.ui.notify('The request to get user location timed out', 'error', 3000);
                break;
            case error.UNKNOWN_ERROR:
                DevExpress.ui.notify('An unknown error occurred', 'error', 3000);
                break;
        }
    }

    function onPhotoDataSuccess(imageData) {
        saveImage(imageData);
    }

    function onPhotoDataFail(message) {
        DevExpress.ui.notify(message, 'error', 3000);
    }

    function saveImage(imageData) {
        IntelliServe_Dxtreme.dbDataLayer.Instance().InsertJobCardImage(id, imageData).done(function () {
            DevExpress.ui.notify('Image successfully uploaded', 'success', 3000);
        })
        .fail(function () {
            DevExpress.ui.notify('Failed uploading image', 'error', 3000);
        });
    }

    function handleJobCardItemsModification() {
        shouldReload = true;
    }

    function handleJobCardEquipmentsModification() {
        shouldReload = true;
    }

    showActionSheet = function () {
        actionSheetVisible(true);
    };

    function hanledJobLog() {
        IntelliServe_Dxtreme.app.navigate(joblogaction());
    }

    function handleViewDisposing() {
        IntelliServe_Dxtreme.db.JobCardItems.off("modified", handleJobCardItemsModification);
        IntelliServe_Dxtreme.db.JobCardEquipments.off("modified", handleJobCardEquipmentsModification);
    }

    dataSourceItems = IntelliServe_Dxtreme.dbDataLayer.Instance().GetJobcardItems(id);
    dataSourceEquipments = IntelliServe_Dxtreme.dbDataLayer.Instance().GetJobcardEquipments(id);

    IntelliServe_Dxtreme.db.JobCardItems.on("modified", handleJobCardItemsModification);

    IntelliServe_Dxtreme.db.JobCardEquipments.on("modified", handleJobCardEquipmentsModification);

    function refreshList() {
        shouldReload = false;
        dataSourceItems.pageIndex(0);
        dataSourceItems.load();

        dataSourceEquipments.pageIndex(0);
        dataSourceEquipments.load();
    }

    return {
        id: id,
        hanledJobLog: hanledJobLog,
        dataSourceItems: dataSourceItems,
        dataSourceEquipments: dataSourceEquipments,
        joblogactiontext: joblogactiontext,
        v_jobcards_mobile: v_jobcards_mobile,
        viewShowing: handleViewShowing,
        actionSheetVisible: actionSheetVisible,
        actionSheetData: actionSheetData,
        showActionSheet: showActionSheet,
        viewDisposing: handleViewDisposing,
        isReady: isReady.promise(),
        GetColor: function (item) {
            var colorObject = pusher.color('packed_argb', item);

            return colorObject.html('hex6');
        }
    };
};