IntelliServe_Dxtreme.QuoteEdit = function(params, viewInfo) {
    "use strict";

    var id = params.id,
        isNew = (id === undefined),
        isSplitLayout = viewInfo.layoutController.name === "split",
        quote = new IntelliServe_Dxtreme.QuoteViewModel(),
        isReady = $.Deferred();

    function load() {
        return IntelliServe_Dxtreme.db.Quotes.byKey(id, { expand: ["User1", "Client1", "Location1", "Invoice1", "JobCard1", "Ticket1", "JobProject1", "QuoteTerm"] }).done(function(data) {
            quote.fromJS(data);
        });
    }

    function update() {
        IntelliServe_Dxtreme.db.Quotes.update(id, quote.toJS()).done(function() {
            IntelliServe_Dxtreme.app.back();
        });
    }

    function insert() {
        IntelliServe_Dxtreme.db.Quotes.insert(quote.toJS()).done(function(values, newId) {
            IntelliServe_Dxtreme.app.navigate({ view: "QuoteDetails", id: newId }, { target: "current" });
        });
    }

    function handleSave() {
        if(isNew)
            insert();
        else
            update();
    }

    function handleCancel() {
        if(!isNew) {
            IntelliServe_Dxtreme.app.back();
        }
        else {
            if(isSplitLayout) {
                IntelliServe_Dxtreme.app.navigate("Blank", { target: "current" });
            }
            else {
                IntelliServe_Dxtreme.app.back();
            }
        }
    }

    function handleViewShowing() {
        quote.clear();
        if(!isNew)
            load().done(function() {
                isReady.resolve();
            });
        else {
            isReady.resolve();
        }
    }

    return {
        quote: quote,
        usersSource: {
            store: IntelliServe_Dxtreme.db.Users,
            map: function(data) {
                return new IntelliServe_Dxtreme.UserViewModel(data);
            }
        },
        clientsSource: {
            store: IntelliServe_Dxtreme.db.Clients,
            map: function(data) {
                return new IntelliServe_Dxtreme.ClientViewModel(data);
            }
        },
        locationsSource: {
            store: IntelliServe_Dxtreme.db.Locations,
            map: function(data) {
                return new IntelliServe_Dxtreme.LocationViewModel(data);
            }
        },
        invoicesSource: {
            store: IntelliServe_Dxtreme.db.Invoices,
            map: function(data) {
                return new IntelliServe_Dxtreme.InvoiceViewModel(data);
            }
        },
        jobcardsSource: {
            store: IntelliServe_Dxtreme.db.JobCards,
            map: function(data) {
                return new IntelliServe_Dxtreme.JobCardViewModel(data);
            }
        },
        ticketsSource: {
            store: IntelliServe_Dxtreme.db.Tickets,
            map: function(data) {
                return new IntelliServe_Dxtreme.TicketViewModel(data);
            }
        },
        jobprojectsSource: {
            store: IntelliServe_Dxtreme.db.JobProjects,
            map: function(data) {
                return new IntelliServe_Dxtreme.JobProjectViewModel(data);
            }
        },
        quotetermsSource: {
            store: IntelliServe_Dxtreme.db.QuoteTerms,
            map: function(data) {
                return new IntelliServe_Dxtreme.QuoteTermViewModel(data);
            }
        },
        handleSave: handleSave,
        handleCancel: handleCancel,
        viewShowing: handleViewShowing,
        isReady: isReady.promise()
    };
};